#!/usr/bin/perl
# -----------------------------------------------------------------------------
# Programm         : C D t o o l s . p m
# -----------------------------------------------------------------------------
# Autor            : CD
# Erstellt         : 27. November 2002
# Zuletzt gendert : 26. November 2003
# Aufgabe          : Verschiedene Tools.
# -----------------------------------------------------------------------------
#
#  $Author: usr207 $
#
#  $Date: 2003/11/26 12:47:32 $
#
#  $Id: CDtools.pm,v 1.6 2003/11/26 12:47:32 usr207 Exp $
#
# -----------------------------------------------------------------------------
#
#  $Log: CDtools.pm,v $
#  Revision 1.6  2003/11/26 12:47:32  usr207
#  mod: Datuns- und Zeitfunktionen entfernt, diese sind nun in CDdatum.pm zu finden.
#
#  Revision 1.5  2003/09/03 07:41:09  usr207
#  Stoppuhrausgaben jetzt auf STDERR
#
#  Revision 1.4  2003/06/16 09:20:16  usr207
#  mod: Einrckungen
#
#  Revision 1.3  2003/01/21 14:42:45  usr207
#  new: zufalls_array_disjunkt
#       stoppuhrstart
#       stoppuhrende
#       BEGIN
#
#  Revision 1.2  2003/01/15 16:48:04  usr207
#  new: zeit_zu_minuten
#       datum_zulaessig
#       min_zu_hmin
#
#  Revision 1.1  2002/12/10 12:55:56  usr207
#  ins CVS eingestellt
#
#
# -----------------------------------------------------------------------------

package CDtools;

# -----------------------------------------------------------------------------
# Von Exporter erben:
# -----------------------------------------------------------------------------
use Exporter;
our @ISA = ('Exporter'); # Inherit from Exporter
our @EXPORT_OK = ('taupu',
                  'zufalls_array_disjunkt',
                  'stoppuhrstart',
                  'stoppuhrende',
                 );

sub taupu ($) {
# -----------------------------------------------------------------------------
# sub          : t a u p u
# -----------------------------------------------------------------------------
# Autor        : CD
# Aufgabe      : Zahlen mit Tausendertrenner (".") ausgeben und Komma als
#                Dezimaltrenner verwenden.
#                (Kopiert aus dem Perl-Cook-Book und angepasst.)
# Parameter    : Auszugebende Zahl
# Rckgabewert : Zahl mit Punkten
# -----------------------------------------------------------------------------
# 0.0.1 - 10.12.2002 - CD - Erstellt
# -----------------------------------------------------------------------------

    my $text = reverse $_[0];
    $text =~ s:\.:,:g; # deutsches Komma als Dezimaltrenner
    $text =~ s/(\d\d\d)(?=\d)(?!\d*,)/$1./g;
    return scalar reverse $text;
} # sub taupu


sub zufalls_array_disjunkt ($$$) {
# -----------------------------------------------------------------------------
# sub          : z u f a l l s _ a r r a y _ d i s j u n k t
# -----------------------------------------------------------------------------
# Autor        : CD
# Aufgabe      : Es wird eine sortierte Menge von $anzahl Zahlen erzeugt, die
#                im Intervall [$min, $max] liegen (also inklusive der Grenzen).
# Parameter    : Anzahl, Minimum, Maximum
# Rckgabewert : Sortierte Menge an Zufallszahlen
# -----------------------------------------------------------------------------
# 0.0.1 - 21.01.2003 - CD - Erstellt
# -----------------------------------------------------------------------------

    my $anzahl = shift; # Anzahl Werte
    my $min    = shift; # Kleinster annehmbarer Wert (inklusive)
    my $max    = shift; # Grter annehmbarer Wert (inklusive)
    my @menge  = ();
    my %zahlen = ();


    die "Der Bereich ($min .. $max) ist zu klein fr $anzahl disjunkte " .
        "Werte!\n" if $max-$min+1<$anzahl;

    for my $i (1 .. $anzahl) {
        my $habschon = 0;
        do {
            my $zahl = int($min + rand($max-$min+1));
            if (defined $zahlen{$zahl}) {
                $habschon = 1;
            }
            else {
                $habschon = 0;
                $zahlen{$zahl} = 1;
            }
        } while $habschon;
    }

    for my $z(sort {$a <=> $b} keys %zahlen) { push @menge, $z }


    return @menge

} # sub zufalls_array_disjunkt


{ # Stoppuhrfunktionen und -Variable:

# -----------------------------------------------------------------------------
# Modulglobale Variable nur fr die Stoppuhr:
# -----------------------------------------------------------------------------
my $Stoppuhrstart = 0;

sub stoppuhrstart () {
# -----------------------------------------------------------------------------
# sub          : s t o p p u h r s t a r t
# -----------------------------------------------------------------------------
# Autor        : CD
# Aufgabe      : Startet die Stoppuhr. (Im Moment kann nur eine gleichzeitig
#                laufen!
#                Wenn mehr gebraucht werden muss das in eine Klasse wandern.)
# Parameter    : keine
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 21.01.2003 - CD - Erstellt
# -----------------------------------------------------------------------------

    $Stoppuhrstart = time;
} # sub stoppuhrstart


sub stoppuhrende ($$) {
# -----------------------------------------------------------------------------
# sub          : s t o p p u h r e n d e
# -----------------------------------------------------------------------------
# Autor        : CD
# Aufgabe      : Hlt die Stoppuhr an. (Im Moment kann nur eine gleichzeitig
#                laufen!
#                Wenn mehr gebraucht werden muss das in eine Klasse wandern.)
# Parameter    : keine
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 21.01.2003 - CD - Erstellt
# 0.0.2 - 02.09.2003 - CD - Ausgabe auf STDERR statt auf STDOUT.
# -----------------------------------------------------------------------------

    my $text  = shift;
    my $dauer = time - $Stoppuhrstart;
    my $secs  = $dauer % 60;
    $dauer   -= $secs;
    $dauer   /= 60;
    my $mins  = $dauer % 60;
    $dauer   -= $mins;
    $dauer   /= 60;
    my $hours = $dauer;
    printf STDERR "$text dauerte %02d:%02d:%02d\n", $hours, $mins, $secs;
} # sub stoppuhrende


} # Ende Einklammerung der Stoppuhrfunktionen


BEGIN {
# -----------------------------------------------------------------------------
# sub          : B E G I N
# -----------------------------------------------------------------------------
# Autor        : CD
# Aufgabe      : Initialisierungen zu Beginn des Paketes
# Parameter    : keine
# Rckgabewert : keiner
# -----------------------------------------------------------------------------
# 0.0.1 - 21.01.2003 - CD - Erstellt
# -----------------------------------------------------------------------------
    srand (time() ^ ($$ + ($$ << 15)));
} # BEGIN

return 1;
